#!/bin/bash
##################################################
#
#                   vmail.sh 
# sprava virtualnich uzivatelu pro postovni server
#       http://jiri.jozif.googlepages.com
#            verze 1.1 (11.4.2008)
#
# skript kontroluje pouze syntaxi konfiguracniho
# souboru, ne logiku
#
##################################################

# pro ladeni, je-li true neprovede se zmena konfigurace mail serveru
debug=false

# kde je imlicitni konfiguracni soubor
confname="vmail.conf"

# pouzivat maildir (jinak mbox)
maildir=true

# implicitni kodovani hesla (z vypisu 'dovecotpw -l')
defaultPasswordScheme="PLAIN"

# kde je co ulozene
vdomains="/etc/postfix/vdomains"
vmailboxes="/etc/postfix/vmailboxes"
valiases="/etc/postfix/valiases"

# passwd pro Dovecot
vpasswd="/etc/dovecot/passwd"

# ucty uvedene v $baseMails budou v kazde domene jako alias, smerujici na $baseUser.
baseUser=""
baseMails=( postmaster abuse )

# program pro kodovani hesla
dovecotpw="/usr/sbin/dovecotpw"

# program postmap
postmap="/usr/sbin/postmap"

# znovunacteni kofigurace pro Dovecot
initDovecot="/etc/init.d/dovecot"
# parametr "reload", "force-reload", ...
initDovecotParam="reload"

# dal uz radeji nic nemenit ...
##################################################
# regularni vyrazy
# user je z pismen, cislic, '_', '.', '-' a zacina pismenem
rexUser=[[:alpha:]][[:alnum:]_\.\-]*
# domena je z pismen, cislic '-' a '.', konci '.' a 2 az 6 pismeny (.museum, .travel)
rexDomain=([[:alnum:]\-]\+\.)\+[[:alpha:]]\{2,6\}

rUser=^\($rexUser\)$
rDomain=^@\($rexDomain\)$
rEmail=^\($rexUser\)@\($rexDomain\)$
rUserPassword=^\($rexUser\)[\*]\(.\+\)$
rUserAlias=^\($rexUser\):\(.\+\)$
rLocalUser=^\"\($rexUser\)\"$
# ve slozenych zavorkach dovoleny pouze VELKA pismena, cislice a znak '-'
rPasswordScheme=^[{][[:upper:][:digit:]\-]\+[}].\+$

function fatalError()
{
     [ "$1" ] && echo "$1"
     exit 1
}

# konfiguracni soubor byl zadan jako parametr
[ $1 ] && confname=$1

[ $UID -ne 0 ] && fatalError "This script must be run by root only"
[ -s $confname ] || fatalError "$confname: file not found or empty"
[ -x $postmap ] || fatalError "$postmap: command not found or not executable"
[ -x $dovecotpw ] || fatalError "$dovecotpw: command not found or not executable"
[ -x $initDovecot ] || fatalError "$initDovecot: command not found or not executable"

cat /dev/null > $vdomains.tmp
cat /dev/null > $vmailboxes.tmp
cat /dev/null > $valiases.tmp
cat /dev/null > $vpasswd.tmp

error=false
iline=1
while read line;
do
     if [[ ! $line == "" ]] && [[ ! $line =~ \# ]] ; then # prazdne radky a poznamky vynechany
          if [[ "$line" =~ $rDomain ]] ; then # aktualni domena
               domain=${BASH_REMATCH[1]}
               echo "$domain"
               echo -e "$domain\t$domain" >> $vdomains.tmp
               if [ $baseUser ] ; then  # doplneni o zakladni adresy
                    for m in ${baseMails[*]}
                    do
                         echo -e "$m@$domain\t$baseUser" >> $valiases.tmp
                    done
               fi                    
          elif [[ "$line" =~ $rUserPassword ]] ; then # user a heslo
               user=${BASH_REMATCH[1]}
               text=${BASH_REMATCH[2]}
               if [[ "$text" =~ $rPasswordScheme ]] ; then # je-li kodovani jiz vyplnene, netreba kodovat heslo
                    password=$text
               else
                    password=$($dovecotpw -p $text -s $defaultPasswordScheme)
                    [ $? -ne 0 ] && fatalError
               fi
               echo "$user@$domain:$password" >> $vpasswd.tmp
               end=""
               if $maildir ; then
                    end="/"
               fi
               echo -e "$user@$domain\t$user@$domain$end" >> $vmailboxes.tmp
          elif [[ "$line" =~ $rUserAlias ]] ; then # postovni aliasy
               alias=${BASH_REMATCH[1]}
               text=${BASH_REMATCH[2]}
               # v $text jsou jednotlive adresy oddelene ',' nebo ' ', ty bez '@' je nutne doplnit o aktualni domenu.
               # nejdrive ',' zamenim na ' ', prevedu na pole, to cele projedu a pripadne upravim
               # jednotlive adresy, nakonec pole dam do stringu a ' ' nahradim ','
               array=($(echo $text | sed 's/,/ /g'))
               [ $? -ne 0 ] && fatalError
               for i in ${!array[@]}
               do
                    if [[ "${array[$i]}" =~ $rEmail ]] ; then # nemusim delat nic
                         :
                    elif [[ "${array[$i]}" =~ $rUser ]] ; then # nutno pridat aktualni domenu
                         array[$i]=${array[$i]}@$domain
                    elif [[ "${array[$i]}" =~ $rLocalUser ]] ; then # lokalni uzivatel, odstranit uvozovky
                         array[$i]=${BASH_REMATCH[1]}
                    else # spatny format adresy
                         echo "error in $confname line $iline, alias: $alias, target: ${array[$i]}"
                         error=true
                    fi  
               done
               aliases=($(echo ${array[*]} | sed 's/ /,/g'))
               [ $? -ne 0 ] && fatalError
               echo -e "$alias@$domain\t$aliases" >> $valiases.tmp
          else # radek nepochopen, chyba
               echo "error in $confname line $iline: $line"          
               error=true
          fi         
     fi
     iline=$((iline + 1))
done < $confname

$error && exit 1

$debug && exit 0

echo "changing configuration mail server"

mv $vdomains.tmp $vdomains
mv $vmailboxes.tmp $vmailboxes
mv $valiases.tmp $valiases
mv $vpasswd.tmp $vpasswd
$postmap hash:$vdomains
$postmap hash:$vmailboxes
$postmap hash:$valiases
$initDovecot $initDovecotParam

echo "done"

exit 0


